@@ -6,52 +6,39 @@
  * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
  * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
  *
- * Copyright (c) 2004, 2005, 2006, 2007 Olof Naessn and Per Larsson
+ * Copyright (c) 2004 - 2008 Olof Naessn and Per Larsson
  *
- *                                                         Js_./
- * Per Larsson a.k.a finalman                          _RqZ{a<^_aa
- * Olof Naessn a.k.a jansem/yakslem                _asww7!uY`>  )\a//
- *                                                 _Qhm`] _f "'c  1!5m
- * Visit: http://guichan.darkbits.org             )Qk<P ` _: :+' .'  "{[
- *                                               .)j(] .d_/ '-(  P .   S
- * License: (BSD)                                <Td/Z <fP"5(\"??"\a.  .L
- * Redistribution and use in source and          _dV>ws?a-?'      ._/L  #'
- * binary forms, with or without                 )4d[#7r, .   '     )d`)[
- * modification, are permitted provided         _Q-5'5W..j/?'   -?!\)cam'
- * that the following conditions are met:       j<<WP+k/);.        _W=j f
- * 1. Redistributions of source code must       .$%w\/]Q  . ."'  .  mj$
- *    retain the above copyright notice,        ]E.pYY(Q]>.   a     J@\
- *    this list of conditions and the           j(]1u<sE"L,. .   ./^ ]{a
- *    following disclaimer.                     4'_uomm\.  )L);-4     (3=
- * 2. Redistributions in binary form must        )_]X{Z('a_"a7'<a"a,  ]"[
- *    reproduce the above copyright notice,       #}<]m7`Za??4,P-"'7. ).m
- *    this list of conditions and the            ]d2e)Q(<Q(  ?94   b-  LQ/
- *    following disclaimer in the                <B!</]C)d_, '(<' .f. =C+m
- *    documentation and/or other materials      .Z!=J ]e []('-4f _ ) -.)m]'
- *    provided with the distribution.          .w[5]' _[ /.)_-"+?   _/ <W"
- * 3. Neither the name of Guichan nor the      :$we` _! + _/ .        j?
- *    names of its contributors may be used     =3)= _f  (_yQmWW$#(    "
- *    to endorse or promote products derived     -   W,  sQQQQmZQ#Wwa]..
- *    from this software without specific        (js, \[QQW$QWW#?!V"".
- *    prior written permission.                    ]y:.<\..          .
- *                                                 -]n w/ '         [.
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT       )/ )/           !
- * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY         <  (; sac    ,    '
- * EXPRESS OR IMPLIED WARRANTIES, INCLUDING,               ]^ .-  %
- * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            c <   r
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR            aga<  <La
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE          5%  )P'-3L
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR        _bQf` y`..)a
- * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,          ,J?4P'.P"_(\?d'.,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES               _Pa,)!f/<[]/  ?"
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT      _2-..:. .r+_,.. .
- * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     ?a.<%"'  " -'.a_ _,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION)                     ^
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
- * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Per Larsson a.k.a finalman
+ * Olof Naessn a.k.a jansem/yakslem
+ *
+ * Visit: http://guichan.sourceforge.net
+ *
+ * License: (BSD)
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ * 3. Neither the name of Guichan nor the names of its contributors may
+ *    be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+ * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 /*
@@ -148,10 +135,8 @@ namespace gcn
             throw GCN_EXCEPTION("No top widget set");
         }
 
-        // Release of modal focus or modal mouse input focus might make it
-        // necessary to distribute mouse events.
-        handleModalFocusRelease();
-        handleModalMouseInputFocusRelease();
+        handleModalFocus();
+        handleModalMouseInputFocus();
 
         if (mInput != NULL)
         {
@@ -183,17 +168,17 @@ namespace gcn
 
         mGraphics->_beginDraw();
 
-        // If top has a border,
+        // If top has a frame,
         // draw it before drawing top
-        if (mTop->getBorderSize() > 0)
+        if (mTop->getFrameSize() > 0)
         {
             Rectangle rec = mTop->getDimension();
-            rec.x -= mTop->getBorderSize();
-            rec.y -= mTop->getBorderSize();
-            rec.width += 2 * mTop->getBorderSize();
-            rec.height += 2 * mTop->getBorderSize();
+            rec.x -= mTop->getFrameSize();
+            rec.y -= mTop->getFrameSize();
+            rec.width += 2 * mTop->getFrameSize();
+            rec.height += 2 * mTop->getFrameSize();
             mGraphics->pushClipArea(rec);
-            mTop->drawBorder(mGraphics);
+            mTop->drawFrame(mGraphics);
             mGraphics->popClipArea();
         }
 
@@ -345,67 +330,153 @@ namespace gcn
     void Gui::handleMouseMoved(const MouseInput& mouseInput)
     {
         // Check if the mouse leaves the application window.
-        if (mFocusHandler->getLastWidgetWithMouse() != NULL
+        if (!mWidgetWithMouseQueue.empty()
             && (mouseInput.getX() < 0
                 || mouseInput.getY() < 0
                 || !mTop->getDimension().isPointInRect(mouseInput.getX(), mouseInput.getY()))
             )
         {
-            int lastWidgetWithMouseX, lastWidgetWithMouseY;
-            mFocusHandler->getLastWidgetWithMouse()->getAbsolutePosition(lastWidgetWithMouseX, lastWidgetWithMouseY);
+            // Distribute an event to all widgets in the "widget with mouse" queue.
+            while (!mWidgetWithMouseQueue.empty())
+            {
+                Widget* widget = mWidgetWithMouseQueue.front();
 
-            distributeMouseEvent(mFocusHandler->getLastWidgetWithMouse(),
-                                 MouseEvent::EXITED,
-                                 mouseInput.getButton(),
-                                 mouseInput.getX(),
-                                 mouseInput.getX(),
-                                 true,
-                                 true);
+                if (Widget::widgetExists(widget))
+                {
+                    distributeMouseEvent(widget,
+                                         MouseEvent::EXITED,
+                                         mouseInput.getButton(),
+                                         mouseInput.getX(),
+                                         mouseInput.getY(),
+                                         true,
+                                         true);
+                }
 
-            mFocusHandler->setLastWidgetWithMouse(NULL);
+                mWidgetWithMouseQueue.pop_front();
+            }
 
             return;
         }
 
-        Widget* sourceWidget = getMouseEventSource(mouseInput.getX(), mouseInput.getY());
-
-        if (sourceWidget != mFocusHandler->getLastWidgetWithMouse())
+        // Check if there is a need to send mouse exited events by
+        // traversing the "widget with mouse" queue.
+        bool widgetWithMouseQueueCheckDone = mWidgetWithMouseQueue.empty();
+        while (!widgetWithMouseQueueCheckDone)
         {
-            if (mFocusHandler->getLastWidgetWithMouse() != NULL)
+            unsigned int iterations = 0;
+            std::deque<Widget*>::iterator iter;
+            for (iter = mWidgetWithMouseQueue.begin();
+                 iter != mWidgetWithMouseQueue.end();
+                 iter++)
             {
-                int lastWidgetWithMouseX, lastWidgetWithMouseY;
-                mFocusHandler->getLastWidgetWithMouse()->getAbsolutePosition(lastWidgetWithMouseX, lastWidgetWithMouseY);
+                Widget* widget = *iter;
+                            
+                // If a widget in the "widget with mouse queue" doesn't
+                // exists anymore it should be removed from the queue.
+                if (!Widget::widgetExists(widget))
+                {
+                    mWidgetWithMouseQueue.erase(iter);
+                    break;
+                }
+                else
+                {
+                    int x, y;
+                    widget->getAbsolutePosition(x, y);
 
-                distributeMouseEvent(mFocusHandler->getLastWidgetWithMouse(),
-                                     MouseEvent::EXITED,
+                    if (x > mouseInput.getX()
+                        || y > mouseInput.getY()
+                        || x + widget->getWidth() <= mouseInput.getX()
+                        || y + widget->getHeight() <= mouseInput.getY()
+                        || !widget->isVisible())
+                    {
+                        distributeMouseEvent(widget,
+                                             MouseEvent::EXITED,
+                                             mouseInput.getButton(),
+                                             mouseInput.getX(),
+                                             mouseInput.getY(),
+                                             true,
+                                             true);                                       
+                        mClickCount = 1;
+                        mLastMousePressTimeStamp = 0;
+                        mWidgetWithMouseQueue.erase(iter);
+                        break;
+                    }
+                }
+
+                iterations++;
+            }
+
+            widgetWithMouseQueueCheckDone = iterations == mWidgetWithMouseQueue.size();
+        }
+
+        // Check all widgets below the mouse to see if they are
+        // present in the "widget with mouse" queue. If a widget
+        // is not then it should be added and an entered event should
+        // be sent to it.
+        Widget* parent = getMouseEventSource(mouseInput.getX(), mouseInput.getY());
+        Widget* widget = parent;
+
+        // If a widget has modal mouse input focus then it will
+        // always be returned from getMouseEventSource, but we only wan't to send
+        // mouse entered events if the mouse has actually entered the widget with
+        // modal mouse input focus, hence we need to check if that's the case. If
+        // it's not we should simply ignore to send any mouse entered events.
+        if (mFocusHandler->getModalMouseInputFocused() != NULL
+            && widget == mFocusHandler->getModalMouseInputFocused()
+            && Widget::widgetExists(widget))
+        {
+            int x, y;
+            widget->getAbsolutePosition(x, y);
+
+            if (x > mouseInput.getX()
+                || y > mouseInput.getY()
+                || x + widget->getWidth() <= mouseInput.getX() 
+                || y + widget->getHeight() <= mouseInput.getY())
+            {
+                parent = NULL;
+            }
+        }
+
+        while (parent != NULL)
+        {
+            parent = (Widget*)widget->getParent();
+
+            // Check if the widget is present in the "widget with mouse" queue.
+            bool widgetIsPresentInQueue = false;
+            std::deque<Widget*>::iterator iter;
+            for (iter = mWidgetWithMouseQueue.begin();
+                 iter != mWidgetWithMouseQueue.end();
+                 iter++)
+            {
+                if (*iter == widget)
+                {
+                    widgetIsPresentInQueue = true;
+                    break;
+                }
+            }
+
+            // Widget is not present, send an entered event and add
+            // it to the "widget with mouse" queue.
+            if (!widgetIsPresentInQueue
+                && Widget::widgetExists(widget))
+            {
+                distributeMouseEvent(widget,
+                                     MouseEvent::ENTERED,
                                      mouseInput.getButton(),
                                      mouseInput.getX(),
                                      mouseInput.getY(),
                                      true,
                                      true);
-                mClickCount = 1;
-                mLastMousePressTimeStamp = 0;
+                mWidgetWithMouseQueue.push_front(widget);
             }
 
-            int sourceWidgetX, sourceWidgetY;
-            sourceWidget->getAbsolutePosition(sourceWidgetX, sourceWidgetY);
-
-            distributeMouseEvent(sourceWidget,
-                                 MouseEvent::ENTERED,
-                                 mouseInput.getButton(),
-                                 mouseInput.getX(),
-                                 mouseInput.getY(),
-                                 true,
-                                 true);
-
-            mFocusHandler->setLastWidgetWithMouse(sourceWidget);
+            Widget* swap = widget;
+            widget = parent;
+            parent = (Widget*)swap->getParent();
         }
 
         if (mFocusHandler->getDraggedWidget() != NULL)
         {
-            int draggedWidgetX, draggedWidgetY;
-            mFocusHandler->getDraggedWidget()->getAbsolutePosition(draggedWidgetX, draggedWidgetY);
-
             distributeMouseEvent(mFocusHandler->getDraggedWidget(),
                                  MouseEvent::DRAGGED,
                                  mLastMouseDragButton,
@@ -414,9 +485,7 @@ namespace gcn
         }
         else
         {
-            int sourceWidgetX, sourceWidgetY;
-            sourceWidget->getAbsolutePosition(sourceWidgetX, sourceWidgetY);
-
+            Widget* sourceWidget = getMouseEventSource(mouseInput.getX(), mouseInput.getY());
             distributeMouseEvent(sourceWidget,
                                  MouseEvent::MOVED,
                                  mouseInput.getButton(),
@@ -437,6 +506,13 @@ namespace gcn
         int sourceWidgetX, sourceWidgetY;
         sourceWidget->getAbsolutePosition(sourceWidgetX, sourceWidgetY);
         
+		if ((mFocusHandler->getModalFocused() != NULL
+            && sourceWidget->isModalFocused())
+            || mFocusHandler->getModalFocused() == NULL)
+        {
+            sourceWidget->requestFocus();
+        }
+
         distributeMouseEvent(sourceWidget,
                              MouseEvent::PRESSED,
                              mouseInput.getButton(),
@@ -445,13 +521,6 @@ namespace gcn
 
         mFocusHandler->setLastWidgetPressed(sourceWidget);
         
-        if (mFocusHandler->getModalFocused() != NULL
-            && sourceWidget->hasModalFocus()
-            || mFocusHandler->getModalFocused() == NULL)
-        {
-            sourceWidget->requestFocus();
-        }
-
         mFocusHandler->setDraggedWidget(sourceWidget);
         mLastMouseDragButton = mouseInput.getButton();
 
@@ -553,54 +622,6 @@ namespace gcn
         }
     }
 
-    void Gui::handleModalFocusRelease()
-    {
-        if (mFocusHandler->getLastWidgetWithModalFocus() 
-            != mFocusHandler->getModalFocused())
-        {
-            Widget* sourceWidget = getWidgetAt(mLastMouseX, mLastMouseY);
-
-            if (sourceWidget != mFocusHandler->getLastWidgetWithModalFocus()
-                && mFocusHandler->getLastWidgetWithModalFocus() != NULL)
-            {
-                int sourceWidgetX, sourceWidgetY;
-                sourceWidget->getAbsolutePosition(sourceWidgetX, sourceWidgetY);
-
-                distributeMouseEvent(sourceWidget,
-                                      MouseEvent::ENTERED,
-                                      mLastMousePressButton,
-                                      mLastMouseX,
-                                      mLastMouseY);
-            }
-
-            mFocusHandler->setLastWidgetWithModalFocus(mFocusHandler->getModalFocused());
-        }
-    }
-
-    void Gui::handleModalMouseInputFocusRelease()
-    {
-        if (mFocusHandler->getLastWidgetWithModalMouseInputFocus() 
-            != mFocusHandler->getModalMouseInputFocused())
-        {
-            Widget* sourceWidget = getWidgetAt(mLastMouseX, mLastMouseY);
-
-            if (sourceWidget != mFocusHandler->getLastWidgetWithModalMouseInputFocus() 
-                && mFocusHandler->getLastWidgetWithModalMouseInputFocus()  != NULL)
-            {
-                int sourceWidgetX, sourceWidgetY;
-                sourceWidget->getAbsolutePosition(sourceWidgetX, sourceWidgetY);
-
-                distributeMouseEvent(sourceWidget,
-                                     MouseEvent::ENTERED,
-                                     mLastMousePressButton,
-                                     mLastMouseX,
-                                     mLastMouseY);
-            }
-
-            mFocusHandler->setLastWidgetWithModalMouseInputFocus(mFocusHandler->getModalMouseInputFocused());
-        }
-    }
-
     Widget* Gui::getWidgetAt(int x, int y)
     {
         // If the widget's parent has no child then we have found the widget..
@@ -624,7 +645,7 @@ namespace gcn
         Widget* widget = getWidgetAt(x, y);
 
         if (mFocusHandler->getModalMouseInputFocused() != NULL
-            && !widget->hasModalMouseInputFocus())
+            && !widget->isModalMouseInputFocused())
         {
             return mFocusHandler->getModalMouseInputFocused();
         }
@@ -657,17 +678,30 @@ namespace gcn
         Widget* widget = source;
 
         if (mFocusHandler->getModalFocused() != NULL
-            && !widget->hasModalFocus())
+            && !widget->isModalFocused()
+            && !force)
         {
             return;
         }
 
         if (mFocusHandler->getModalMouseInputFocused() != NULL
-            && !widget->hasModalMouseInputFocus())
+            && !widget->isModalMouseInputFocused()
+            && !force)
         {
             return;
         }
 
+        MouseEvent mouseEvent(source,
+                              mShiftPressed,
+                              mControlPressed,
+                              mAltPressed,
+                              mMetaPressed,
+                              type,
+                              button,
+                              x,
+                              y,
+                              mClickCount);
+
         while (parent != NULL)
         {
             // If the widget has been removed due to input
@@ -684,16 +718,8 @@ namespace gcn
                 int widgetX, widgetY;
                 widget->getAbsolutePosition(widgetX, widgetY);
 
-                MouseEvent mouseEvent(source,
-                                      mShiftPressed,
-                                      mControlPressed,
-                                      mAltPressed,
-                                      mMetaPressed,
-                                      type,
-                                      button,
-                                      x - widgetX,
-                                      y - widgetY,
-                                      mClickCount);
+                mouseEvent.mX = x - widgetX;
+                mouseEvent.mY = y - widgetY;
                                       
                 std::list<MouseListener*> mouseListeners = widget->_getMouseListeners();
 
@@ -750,7 +776,7 @@ namespace gcn
             // If a non modal focused widget has been reach
             // and we have modal focus cancel the distribution.
             if (mFocusHandler->getModalFocused() != NULL
-                && !widget->hasModalFocus())
+                && !widget->isModalFocused())
             {
                 break;
             }
@@ -758,7 +784,7 @@ namespace gcn
             // If a non modal mouse input focused widget has been reach
             // and we have modal mouse input focus cancel the distribution.
             if (mFocusHandler->getModalMouseInputFocused() != NULL
-                && !widget->hasModalMouseInputFocus())
+                && !widget->isModalMouseInputFocused())
             {
                 break;
             }
@@ -771,13 +797,13 @@ namespace gcn
         Widget* widget = keyEvent.getSource();
 
         if (mFocusHandler->getModalFocused() != NULL
-            && !widget->hasModalFocus())
+            && !widget->isModalFocused())
         {
             return;
         }
 
         if (mFocusHandler->getModalMouseInputFocused() != NULL
-            && !widget->hasModalMouseInputFocus())
+            && !widget->isModalMouseInputFocused())
         {
             return;
         }
@@ -823,7 +849,7 @@ namespace gcn
             // If a non modal focused widget has been reach
             // and we have modal focus cancel the distribution.
             if (mFocusHandler->getModalFocused() != NULL
-                && !widget->hasModalFocus())
+                && !widget->isModalFocused())
             {
                 break;
             }
@@ -854,4 +880,116 @@ namespace gcn
             }
         }
     }
+
+    void Gui::handleModalMouseInputFocus()
+    {
+        // Check if modal mouse input focus has been gained by a widget.
+        if ((mFocusHandler->getLastWidgetWithModalMouseInputFocus() 
+                != mFocusHandler->getModalMouseInputFocused())
+             && (mFocusHandler->getLastWidgetWithModalMouseInputFocus() == NULL))
+        {
+            handleModalFocusGained();
+            mFocusHandler->setLastWidgetWithModalMouseInputFocus(mFocusHandler->getModalMouseInputFocused());
+        }
+        // Check if modal mouse input focus has been released.
+        else if ((mFocusHandler->getLastWidgetWithModalMouseInputFocus()
+                    != mFocusHandler->getModalMouseInputFocused())
+                    && (mFocusHandler->getLastWidgetWithModalMouseInputFocus() != NULL))
+        {
+            handleModalFocusReleased();
+            mFocusHandler->setLastWidgetWithModalMouseInputFocus(NULL);
+        }
+    }
+
+     void Gui::handleModalFocus()
+    {
+        // Check if modal focus has been gained by a widget.
+        if ((mFocusHandler->getLastWidgetWithModalFocus() 
+                != mFocusHandler->getModalFocused())
+             && (mFocusHandler->getLastWidgetWithModalFocus() == NULL))
+        {
+            handleModalFocusGained();
+            mFocusHandler->setLastWidgetWithModalFocus(mFocusHandler->getModalFocused());
+        }
+        // Check if modal focus has been released.
+        else if ((mFocusHandler->getLastWidgetWithModalFocus()
+                    != mFocusHandler->getModalFocused())
+                    && (mFocusHandler->getLastWidgetWithModalFocus() != NULL))
+        {
+            handleModalFocusReleased();
+            mFocusHandler->setLastWidgetWithModalFocus(NULL);
+        }
+    }
+
+    void Gui::handleModalFocusGained()
+    {
+         // Distribute an event to all widgets in the "widget with mouse" queue.
+        while (!mWidgetWithMouseQueue.empty())
+        {
+            Widget* widget = mWidgetWithMouseQueue.front();
+
+            if (Widget::widgetExists(widget))
+            {
+                distributeMouseEvent(widget,
+                                     MouseEvent::EXITED,
+                                     mLastMousePressButton,
+                                     mLastMouseX,
+                                     mLastMouseY,
+                                     true,
+                                     true);
+            }
+
+            mWidgetWithMouseQueue.pop_front();
+        }
+
+        mFocusHandler->setLastWidgetWithModalMouseInputFocus(mFocusHandler->getModalMouseInputFocused());
+    }
+
+    void Gui::handleModalFocusReleased()
+    {
+         // Check all widgets below the mouse to see if they are
+        // present in the "widget with mouse" queue. If a widget
+        // is not then it should be added and an entered event should
+        // be sent to it.
+        Widget* widget = getMouseEventSource(mLastMouseX, mLastMouseY);
+        Widget* parent = widget;
+
+        while (parent != NULL)
+        {
+            parent = (Widget*)widget->getParent();
+
+            // Check if the widget is present in the "widget with mouse" queue.
+            bool widgetIsPresentInQueue = false;
+            std::deque<Widget*>::iterator iter;
+            for (iter = mWidgetWithMouseQueue.begin();
+                 iter != mWidgetWithMouseQueue.end();
+                 iter++)
+            {
+                if (*iter == widget)
+                {
+                    widgetIsPresentInQueue = true;
+                    break;
+                }
+            }
+
+            // Widget is not present, send an entered event and add
+            // it to the "widget with mouse" queue.
+            if (!widgetIsPresentInQueue
+                && Widget::widgetExists(widget))
+            {
+                distributeMouseEvent(widget,
+                                     MouseEvent::ENTERED,
+                                     mLastMousePressButton,
+                                     mLastMouseX,
+                                     mLastMouseY,
+                                     false,
+                                     true);
+                mWidgetWithMouseQueue.push_front(widget);
+            }
+
+            Widget* swap = widget;
+            widget = parent;
+            parent = (Widget*)swap->getParent();
+        }
+    }
 }
