@@ -6,52 +6,39 @@
  * /______/ //______/ //_/ //_____/\ /_/ //_/ //_/ //_/ //_/ /|_/ /
  * \______\/ \______\/ \_\/ \_____\/ \_\/ \_\/ \_\/ \_\/ \_\/ \_\/
  *
- * Copyright (c) 2004, 2005, 2006, 2007 Olof Naessn and Per Larsson
+ * Copyright (c) 2004 - 2008 Olof Naessn and Per Larsson
  *
- *                                                         Js_./
- * Per Larsson a.k.a finalman                          _RqZ{a<^_aa
- * Olof Naessn a.k.a jansem/yakslem                _asww7!uY`>  )\a//
- *                                                 _Qhm`] _f "'c  1!5m
- * Visit: http://guichan.darkbits.org             )Qk<P ` _: :+' .'  "{[
- *                                               .)j(] .d_/ '-(  P .   S
- * License: (BSD)                                <Td/Z <fP"5(\"??"\a.  .L
- * Redistribution and use in source and          _dV>ws?a-?'      ._/L  #'
- * binary forms, with or without                 )4d[#7r, .   '     )d`)[
- * modification, are permitted provided         _Q-5'5W..j/?'   -?!\)cam'
- * that the following conditions are met:       j<<WP+k/);.        _W=j f
- * 1. Redistributions of source code must       .$%w\/]Q  . ."'  .  mj$
- *    retain the above copyright notice,        ]E.pYY(Q]>.   a     J@\
- *    this list of conditions and the           j(]1u<sE"L,. .   ./^ ]{a
- *    following disclaimer.                     4'_uomm\.  )L);-4     (3=
- * 2. Redistributions in binary form must        )_]X{Z('a_"a7'<a"a,  ]"[
- *    reproduce the above copyright notice,       #}<]m7`Za??4,P-"'7. ).m
- *    this list of conditions and the            ]d2e)Q(<Q(  ?94   b-  LQ/
- *    following disclaimer in the                <B!</]C)d_, '(<' .f. =C+m
- *    documentation and/or other materials      .Z!=J ]e []('-4f _ ) -.)m]'
- *    provided with the distribution.          .w[5]' _[ /.)_-"+?   _/ <W"
- * 3. Neither the name of Guichan nor the      :$we` _! + _/ .        j?
- *    names of its contributors may be used     =3)= _f  (_yQmWW$#(    "
- *    to endorse or promote products derived     -   W,  sQQQQmZQ#Wwa]..
- *    from this software without specific        (js, \[QQW$QWW#?!V"".
- *    prior written permission.                    ]y:.<\..          .
- *                                                 -]n w/ '         [.
- * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT       )/ )/           !
- * HOLDERS AND CONTRIBUTORS "AS IS" AND ANY         <  (; sac    ,    '
- * EXPRESS OR IMPLIED WARRANTIES, INCLUDING,               ]^ .-  %
- * BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF            c <   r
- * MERCHANTABILITY AND FITNESS FOR A PARTICULAR            aga<  <La
- * PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE          5%  )P'-3L
- * COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE FOR        _bQf` y`..)a
- * ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,          ,J?4P'.P"_(\?d'.,
- * EXEMPLARY, OR CONSEQUENTIAL DAMAGES               _Pa,)!f/<[]/  ?"
- * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT      _2-..:. .r+_,.. .
- * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,     ?a.<%"'  " -'.a_ _,
- * DATA, OR PROFITS; OR BUSINESS INTERRUPTION)                     ^
- * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
- * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
- * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
- * ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
- * IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ * Per Larsson a.k.a finalman
+ * Olof Naessn a.k.a jansem/yakslem
+ *
+ * Visit: http://guichan.sourceforge.net
+ *
+ * License: (BSD)
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in
+ *    the documentation and/or other materials provided with the
+ *    distribution.
+ * 3. Neither the name of Guichan nor the names of its contributors may
+ *    be used to endorse or promote products derived from this software
+ *    without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
+ * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
+ * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
+ * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
+ * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
+ * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
+ * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
+ * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
+ * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
+ * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "guichan/widgets/dropdown.hpp"
@@ -104,6 +91,7 @@ namespace gcn
         add(mScrollArea);
 
         mListBox->addActionListener(this);
+        mListBox->addSelectionListener(this);
 
         setListModel(listModel);
 
@@ -117,7 +105,6 @@ namespace gcn
         addFocusListener(this);
 
         adjustHeight();
-        setBorderSize(1);
     }
 
     DropDown::~DropDown()
@@ -125,6 +112,7 @@ namespace gcn
         if (widgetExists(mListBox))
         {
             mListBox->removeActionListener(this);
+            mListBox->removeSelectionListener(this);
         }
 
         if (mInternalScrollArea)
@@ -136,6 +124,8 @@ namespace gcn
         {
             delete mListBox;
         }
+
+        setInternalFocusHandler(NULL);
     }
 
     void DropDown::draw(Graphics* graphics)
@@ -144,77 +134,75 @@ namespace gcn
 
         if (mDroppedDown)
         {
-            h = mOldH;
+            h = mFoldedUpHeight;
         }
         else
         {
             h = getHeight();
         }
 
-        int alpha = getBaseColor().a;
-        Color faceColor = getBaseColor();
-        faceColor.a = alpha;
-        Color highlightColor = faceColor + 0x303030;
-        highlightColor.a = alpha;
-        Color shadowColor = faceColor - 0x303030;
-        shadowColor.a = alpha;
-
-
-        graphics->setColor(getBackgroundColor());
-        graphics->fillRectangle(Rectangle(0, 0, getWidth(), h));
-
-        graphics->setColor(getForegroundColor());
-        graphics->setFont(getFont());
-
-        if (isFocused())
-        {
-            graphics->setColor(getSelectionColor());
-            graphics->fillRectangle(Rectangle(0, 0, getWidth() - h, h));
-            graphics->setColor(getForegroundColor());
-        }
-
-        if (mListBox->getListModel() && mListBox->getSelected() >= 0)
-        {
-            graphics->drawText(mListBox->getListModel()->getElementAt(mListBox->getSelected()), 1, 0);
-        }
-
-        drawButton(graphics);
-
-         if (mDroppedDown)
-         {
-             drawChildren(graphics);
-
-            // Draw two lines separating the ListBox with se selected
-            // element view.
-            graphics->setColor(highlightColor);
-            graphics->drawLine(0, h, getWidth(), h);
-            graphics->setColor(shadowColor);
-            graphics->drawLine(0, h + 1,getWidth(),h + 1);
-         }
-    }
-
-    void DropDown::drawBorder(Graphics* graphics)
-    {
         Color faceColor = getBaseColor();
         Color highlightColor, shadowColor;
         int alpha = getBaseColor().a;
-        int width = getWidth() + getBorderSize() * 2 - 1;
-        int height = getHeight() + getBorderSize() * 2 - 1;
         highlightColor = faceColor + 0x303030;
         highlightColor.a = alpha;
         shadowColor = faceColor - 0x303030;
         shadowColor.a = alpha;
 
-        unsigned int i;
-        for (i = 0; i < getBorderSize(); ++i)
+        // Draw a border.
+        graphics->setColor(shadowColor);
+        graphics->drawLine(0, 0, getWidth() - 1, 0);
+        graphics->drawLine(0, 1, 0, h - 2);
+        graphics->setColor(highlightColor);
+        graphics->drawLine(getWidth() - 1, 1, getWidth() - 1, h - 1);
+        graphics->drawLine(0, h - 1, getWidth() - 1, h - 1);
+
+         // Push a clip area so the other drawings don't need to worry
+        // about the border.
+        graphics->pushClipArea(Rectangle(1, 1, getWidth() - 2, h - 2));
+        const Rectangle currentClipArea = graphics->getCurrentClipArea();
+
+        graphics->setColor(getBackgroundColor());
+        graphics->fillRectangle(Rectangle(0, 0, currentClipArea.width, currentClipArea.height));
+       
+        if (isFocused())
         {
-            graphics->setColor(shadowColor);
-            graphics->drawLine(i,i, width - i, i);
-            graphics->drawLine(i,i + 1, i, height - i - 1);
-            graphics->setColor(highlightColor);
-            graphics->drawLine(width - i,i + 1, width - i, height - i);
-            graphics->drawLine(i,height - i, width - i - 1, height - i);
+            graphics->setColor(getSelectionColor());
+            graphics->fillRectangle(Rectangle(0, 
+                                              0, 
+                                              currentClipArea.width - currentClipArea.height, 
+                                              currentClipArea.height));
+            graphics->setColor(getForegroundColor());
         }
+
+        if (mListBox->getListModel() 
+            && mListBox->getSelected() >= 0)
+        {
+            graphics->setColor(getForegroundColor());
+            graphics->setFont(getFont());
+
+            graphics->drawText(mListBox->getListModel()->getElementAt(mListBox->getSelected()), 1, 0);
+        }
+        
+        // Push a clip area before drawing the button.
+        graphics->pushClipArea(Rectangle(currentClipArea.width - currentClipArea.height,
+                                         0,
+                                         currentClipArea.height,
+                                         currentClipArea.height));
+        drawButton(graphics);
+        graphics->popClipArea();
+        graphics->popClipArea();
+
+         if (mDroppedDown)
+         {
+             // Draw a border around the children.
+             graphics->setColor(shadowColor);
+             graphics->drawRectangle(Rectangle(0,
+                                               mFoldedUpHeight,
+                                               getWidth(),
+                                               getHeight() - mFoldedUpHeight));
+             drawChildren(graphics);
+         }
     }
 
     void DropDown::drawButton(Graphics *graphics)
@@ -244,45 +232,48 @@ namespace gcn
             offset = 0;
         }
 
-        int h;
-        if (mDroppedDown)
-        {
-            h = mOldH;
-        }
-        else
-        {
-            h = getHeight();
-        }
-        int x = getWidth() - h;
-        int y = 0;
+        const Rectangle currentClipArea = graphics->getCurrentClipArea();
+        graphics->setColor(highlightColor);
+        graphics->drawLine(0, 
+                           0, 
+                           currentClipArea.width - 1, 
+                           0);
+        graphics->drawLine(0, 
+                           1, 
+                           0, 
+                           currentClipArea.height - 1);
+        graphics->setColor(shadowColor);
+        graphics->drawLine(currentClipArea.width - 1, 
+                           1, 
+                           currentClipArea.width - 1, 
+                           currentClipArea.height - 1);
+        graphics->drawLine(1, 
+                           currentClipArea.height - 1, 
+                           currentClipArea.width - 2, 
+                           currentClipArea.height - 1);
 
         graphics->setColor(faceColor);
-        graphics->fillRectangle(Rectangle(x+1, y+1, h-2, h-2));
-
-        graphics->setColor(highlightColor);
-        graphics->drawLine(x, y, x+h-1, y);
-        graphics->drawLine(x, y+1, x, y+h-1);
-
-        graphics->setColor(shadowColor);
-        graphics->drawLine(x+h-1, y+1, x+h-1, y+h-1);
-        graphics->drawLine(x+1, y+h-1, x+h-2, y+h-1);
+        graphics->fillRectangle(Rectangle(1, 
+                                          1, 
+                                          currentClipArea.width - 2, 
+                                          currentClipArea.height - 2));
 
         graphics->setColor(getForegroundColor());
 
         int i;
-        int hh = h / 3;
-        int hx = x + h / 2;
-        int hy = y + (h * 2) / 3;
-        for (i=0; i<hh; i++)
+        int n = currentClipArea.height / 3;
+        int dx = currentClipArea.height / 2;
+        int dy = (currentClipArea.height * 2) / 3;
+        for (i = 0; i < n; i++)
         {
-            graphics->drawLine(hx - i + offset,
-                               hy - i + offset,
-                               hx + i + offset,
-                               hy - i + offset);
+            graphics->drawLine(dx - i + offset,
+                               dy - i + offset,
+                               dx + i + offset,
+                               dy - i + offset);
         }
     }
 
-    int DropDown::getSelected()
+    int DropDown::getSelected() const
     {
         return mListBox->getSelected();
     }
@@ -334,7 +325,7 @@ namespace gcn
         }
         // Fold up the listbox if the upper part is clicked after fold down
         else if (0 <= mouseEvent.getY()
-                 && mouseEvent.getY() < mOldH
+                 && mouseEvent.getY() < mFoldedUpHeight
                  && mouseEvent.getX() >= 0
                  && mouseEvent.getX() < getWidth()
                  && mouseEvent.getButton() == MouseEvent::LEFT
@@ -364,12 +355,12 @@ namespace gcn
         }
 
         // Released outside of widget. Can happen when we have modal input focus.
-        if (0 > mouseEvent.getY()
+        if ((0 > mouseEvent.getY()
             || mouseEvent.getY() >= getHeight()
             || mouseEvent.getX() < 0
-            || mouseEvent.getX() >= getWidth()
+            || mouseEvent.getX() >= getWidth())
             && mouseEvent.getButton() == MouseEvent::LEFT
-            && hasModalMouseInputFocus())
+            && isModalMouseInputFocused())
         {
             releaseModalMouseInputFocus();
 
@@ -423,7 +414,9 @@ namespace gcn
         }
 
         int listBoxHeight = mListBox->getHeight();
-        int h2 = getFont()->getHeight();
+        
+        // We add 2 for the border
+        int h2 = getFont()->getHeight() + 2;
 
         setHeight(h2);
 
@@ -457,7 +450,7 @@ namespace gcn
         if (!mDroppedDown)
         {
             mDroppedDown = true;
-            mOldH = getHeight();
+            mFoldedUpHeight = getHeight();
             adjustHeight();
 
             if (getParent())
@@ -500,14 +493,18 @@ namespace gcn
     {
         foldUp();
         releaseModalMouseInputFocus();
-        generateAction();
+        distributeActionEvent();
     }
 
     Rectangle DropDown::getChildrenArea()
     {
         if (mDroppedDown)
         {
-            return Rectangle(0, mOldH + 2, getWidth(), getHeight() - mOldH);
+            // Calculate the children area (with the one pixel border in mind)
+            return Rectangle(1, 
+                             mFoldedUpHeight + 1, 
+                             getWidth() - 2, 
+                             getHeight() - mFoldedUpHeight - 2);
         }
 
         return Rectangle();
@@ -595,5 +592,41 @@ namespace gcn
             mListBox->setSelected(mListBox->getSelected() + 1);
         }
     }
+
+    void DropDown::setSelectionColor(const Color& color)
+    {
+        Widget::setSelectionColor(color);
+        
+        if (mInternalListBox)
+        {
+            mListBox->setSelectionColor(color);
+        }       
+    }
+
+    void DropDown::valueChanged(const SelectionEvent& event)
+    {
+        distributeValueChangedEvent();
+    }
+
+    void DropDown::addSelectionListener(SelectionListener* selectionListener)
+    {
+        mSelectionListeners.push_back(selectionListener);
+    }
+   
+    void DropDown::removeSelectionListener(SelectionListener* selectionListener)
+    {
+        mSelectionListeners.remove(selectionListener);
+    }
+
+    void DropDown::distributeValueChangedEvent()
+    {
+        SelectionListenerIterator iter;
+
+        for (iter = mSelectionListeners.begin(); iter != mSelectionListeners.end(); ++iter)
+        {
+            SelectionEvent event(this);
+            (*iter)->valueChanged(event);
+        }
+    }
 }
 
